#!/usr/bin/python
#
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import click
from lmanage.capturator import commands as cap_commands
from lmanage.configurator import commands as config_commands


@click.group()
@click.version_option()
def lmanage():
    pass


lmanage.add_command(cap_commands.capturator)
lmanage.add_command(config_commands.configurator)
if __name__ == '__main__':
    lmanage()
